/*===--- x86gprintrin.h - Implementation of X86 GPR intrinsics on PowerPC --===
 *
 * Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
 * See https://llvm.org/LICENSE.txt for license information.
 * SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 *
 *===-----------------------------------------------------------------------===
 */

#ifndef X86GPRINTRIN_H_
#define X86GPRINTRIN_H_

#include <bmiintrin.h>

#include <bmi2intrin.h>

#endif /* X86GPRINTRIN_H_ */
